/*
 * Diese Klasse implementiert die Kommunikation mit dem SoundSquare-Server
 * 
 */

package de.lmu.mobilehci.locationaudio;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Vector;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.FileEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;

import android.util.AttributeSet;
import android.util.Log;
import android.util.Xml;

public class Server {

	public final static String SERVER_URL = "http://141.84.8.81:8000";

	
	// Parst Liste von POIs die als XML vom Server zurckgegeben werden
	private Vector<PointOfInterest> parsePoiList(Reader in) throws IOException, XmlPullParserException {
		// XmlResourceParser parser = getResources().getXml(R.xml.testpoilist);

		XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
		XmlPullParser parser = factory.newPullParser();
		parser.setInput(in);
		AttributeSet attributes = Xml.asAttributeSet(parser);

		Vector<PointOfInterest> pois = new Vector<PointOfInterest>();

		parser.next();
		int eventType = parser.getEventType();

		PointOfInterest poi = null;
		int startedId = 0;

		while (eventType != XmlPullParser.END_DOCUMENT) {
			switch (eventType) {
			case XmlPullParser.START_DOCUMENT:
				// Log.d("SoundSquare", "START_DOCUMENT " + parser.getName());
				break;
			case XmlPullParser.START_TAG:
				// debug output
				/*
				 * Log.d("SoundSquare", "START_TAG " + parser.getName()); int n
				 * = parser.getAttributeCount(); Log.d("SoundSquare",
				 * "attribute count = " + n); for (int i = 0; i < n; i++) {
				 * Log.d("SoundSquare", "attribute name = " +
				 * parser.getAttributeName(i) + ", value = " +
				 * parser.getAttributeValue(i)); }
				 */
				String entityName = parser.getName();
				if ("poi".equals(entityName)) {
					poi = new PointOfInterest();
					poi.latitude = attributes.getAttributeFloatValue(null, "lat", 0);
					poi.longitude = attributes.getAttributeFloatValue(null, "lon", 0);
					poi.distance = attributes.getAttributeFloatValue(null, "dist", 0);
					poi.id = attributes.getAttributeIntValue(null, "id", 0);
					pois.add(poi);
				} else if ("name".equals(entityName)) {
					startedId = PointOfInterest.STARTED_NAME;
				} else if ("tag".equals(entityName)) {
					startedId = PointOfInterest.STARTED_TAG;
				} else if ("date".equals(entityName)) {
					startedId = PointOfInterest.STARTED_CREATION_DATE;
				}
				break;
			case XmlPullParser.END_TAG:
				// Log.d("SoundSquare", "END_TAG " + parser.getName());
				if ("poi".equals(parser.getName())) {
					poi = null;
				}
				break;
			case XmlPullParser.TEXT:
				// Log.d("SoundSquare", "TEXT " + parser.getText());
				if (poi != null) {
					if (startedId == PointOfInterest.STARTED_NAME) {
						poi.name = URLDecoder.decode(parser.getText().trim());
						startedId = 0;
					} else if (startedId == PointOfInterest.STARTED_TAG) {
						poi.tag = URLDecoder.decode(parser.getText().trim());
						startedId = 0;
					} else if (startedId == PointOfInterest.STARTED_CREATION_DATE) {
						String ds = parser.getText().trim();
						poi.creationDate = ds;
						/*
						 * DateFormat df = DateFormat.getDateTimeInstance(); try
						 * { Date d = df.parse(ds); poi.creationDate = d; }
						 * catch (ParseException ex) { Log.d("SoundSquare",
						 * "Exception parsing date: " + ex.toString()); }
						 */
						startedId = 0;
					}
				}
				break;
			default:
				// ignore
				break;
			}
			eventType = parser.next();
		}

		return pois;

	}

	
	// Parst Liste von Soundinformationen, die vom Server zurckgegeben wird
	private Vector<SoundInfo> parseSoundList(Reader in) throws IOException, XmlPullParserException {
		// XmlResourceParser parser = getResources().getXml(R.xml.testsoundlist);

		XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
		XmlPullParser parser = factory.newPullParser();
		parser.setInput(in);
		AttributeSet attributes = Xml.asAttributeSet(parser);

		Vector<SoundInfo> sounds = new Vector<SoundInfo>();

		parser.next();
		int eventType = parser.getEventType();

		SoundInfo sound = null;
		int startedId = 0;

		while (eventType != XmlPullParser.END_DOCUMENT) {
			switch (eventType) {
			case XmlPullParser.START_DOCUMENT:
				// Log.d("SoundSquare", "START_DOCUMENT " + parser.getName());
				break;
			case XmlPullParser.START_TAG:
				// debug output
				/*
				 * Log.d("SoundSquare", "START_TAG " + parser.getName()); int n
				 * = parser.getAttributeCount(); Log.d("SoundSquare",
				 * "attribute count = " + n); for (int i = 0; i < n; i++) {
				 * Log.d("SoundSquare", "attribute name = " +
				 * parser.getAttributeName(i) + ", value = " +
				 * parser.getAttributeValue(i)); }
				 */
				String entityName = parser.getName();
				if ("sound".equals(entityName)) {
					sound = new SoundInfo();
					sound.poiId = attributes.getAttributeIntValue(null, "poi_id", 0);
					sound.soundId = attributes.getAttributeIntValue(null, "sound_id", 0);
					sounds.add(sound);
				} else if ("description".equals(entityName)) {
					startedId = SoundInfo.STARTED_DESCRIPTION;
				} else if ("playback_length".equals(entityName)) {
					startedId = SoundInfo.STARTED_PLAYBACK_LENGTH;
				} else if ("date_created".equals(entityName)) {
					startedId = SoundInfo.STARTED_CREATION_DATE;
				} else if ("date_last_played".equals(entityName)) {
					startedId = SoundInfo.STARTED_LAST_PLAYED_DATE;
				} else if ("nr_playbacks".equals(entityName)) {
					startedId = SoundInfo.STARTED_NUMBER_OF_PLAYBACKS;
				} else if ("path".equals(entityName)) {
					startedId = SoundInfo.STARTED_PATH;
				}
				break;
			case XmlPullParser.END_TAG:
				// Log.d("SoundSquare", "END_TAG " + parser.getName());
				if ("sound".equals(parser.getName())) {
					sound = null;
				}
				break;
			case XmlPullParser.TEXT:
				// Log.d("SoundSquare", "TEXT " + parser.getText());
				if (sound != null) {
					if (startedId == SoundInfo.STARTED_DESCRIPTION) {
						sound.description = URLDecoder.decode(parser.getText().trim());
						startedId = 0;
					}
					if (startedId == SoundInfo.STARTED_PLAYBACK_LENGTH) {
						sound.playbackLength = Integer.parseInt(parser.getText().trim());
						startedId = 0;
					}
					if (startedId == SoundInfo.STARTED_CREATION_DATE) {
						sound.creationDate = parser.getText().trim();
						startedId = 0;
					}
					if (startedId == SoundInfo.STARTED_LAST_PLAYED_DATE) {
						sound.lastPlayedDate = parser.getText().trim();
						startedId = 0;
					}
					if (startedId == SoundInfo.STARTED_NUMBER_OF_PLAYBACKS) {
						sound.numberOfPlaybacks = Integer.parseInt(parser.getText().trim());
						startedId = 0;
					}
					if (startedId == SoundInfo.STARTED_PATH) {
						sound.path = parser.getText().trim();
						startedId = 0;
					}
				}
				break;
			default:
				// ignore
				break;
			}
			eventType = parser.next();
		}

		return sounds;

	}

	
	// Diese Methode ruft eine Liste von Pois ab, die sich im Radius "radius" um das
	// Koordinatenpaar "lon" und "lat" befinden.
	// ^soundsquare/getpois/(?P<lat>\S+)/(?P<lon>\S+)/(?P<radius>\S+)/$
	public Vector<PointOfInterest> getPois(double lat, double lon, double radius) {
		Vector<PointOfInterest> pois = null;
		try {
			HttpClient client = new DefaultHttpClient();
			HttpGet request = new HttpGet(SERVER_URL + "/soundsquare/getpois/" + lat + "/" + lon + "/" + radius + "/");
			Log.d("SoundSquare", "getPois, requestURL = " + request.getURI());
			
			HttpResponse response = client.execute(request);
			Log.d("SoundSquare", response.getStatusLine().toString());
			
			BufferedReader in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			pois = parsePoiList(in);
			in.close();
			
			client.getConnectionManager().shutdown();
		} catch (Exception ex) {
			Log.d("SoundSquare", "Exception getPois: " + ex.toString());
		}
		return pois;
	}

	

	

	
	// Mit diesem Aufruf wird eine Sound-Aufnahme mit einem POI verknpft. Die Audiodaten werden per
	// HTTP-POST an den Server bertragen.
	// ^soundsquare/addsoundtopoi/(?P<poiID>\d+)/(?P<desc>.+?)/(?P<length>\d+)/$
	public void addSoundToPoi(SoundInfo si) {
		Log.d("SoundSquare", "addSoundToPoi = " + si.toString());
		try {
			HttpClient client = new DefaultHttpClient();
			String requestURL = SERVER_URL + "/soundsquare/addsoundtopoi/" + 
				si.poiId + "/" + 
				URLEncoder.encode(si.description) + "/" + 
				si.playbackLength + "/";
			Log.d("SoundSquare", "addSoundToPoi, requestURL = " + requestURL);
			HttpPost request = new HttpPost(requestURL);

			// local file stored in si.path
			File f = new File(si.path);
			request.setEntity(new FileEntity(f, "audio/x-mpeg")); // audio/mpeg

			HttpResponse response = client.execute(request);
			Log.d("SoundSquare", response.getStatusLine().toString());

			HttpEntity resEntity = response.getEntity();
			if (resEntity != null) {
				Log.d("SoundSquare", EntityUtils.toString(resEntity));
			}

			client.getConnectionManager().shutdown();

		} catch (Exception ex) {
			Log.d("SoundSquare", "Exception addSoundToPoi: " + ex.toString());
		}
	}

	
	// Diese Methode liefert eine Liste von SoundInfo-Objekten zurck, die mit einem
	// POI verknpft sind. 
	// ^soundsquare/getsounds/(?P<poiID>\d+)/$
	public Vector<SoundInfo> getSounds(PointOfInterest poi) {
		Vector<SoundInfo> sounds = null;
		try {
			HttpClient client = new DefaultHttpClient();
			HttpGet request = new HttpGet(SERVER_URL + "/soundsquare/getsounds/" + poi.id + "/");
			Log.d("SoundSquare", "getSounds, requestURL = " + request.getURI());

			HttpResponse response = client.execute(request);
			
			BufferedReader in = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
			sounds = parseSoundList(in);
			in.close();

			client.getConnectionManager().shutdown();
		} catch (Exception ex) {
			Log.d("SoundSquare", "Exception getSounds: " + ex.toString());
		}
		return sounds;
	}

	
	// Mit dieser Methode lsst sich auf dem Server ein neuer POI anlegen
	// ^soundsquare/addpoi/(?P<lat>\S+)/(?P<lon>\S+)/(?P<name_>.+?)/(?P<tag_>\S+)/$
	public void addPoi(PointOfInterest poi) {
		Log.d("SoundSquare", "addPoi");
		try {
			HttpClient client = new DefaultHttpClient();
			String requestURL = SERVER_URL + "/soundsquare/addpoi/" + 
					poi.latitude + "/" + 
					poi.longitude + "/" + 
					URLEncoder.encode(poi.name) + "/" + 
					URLEncoder.encode(poi.tag) + "/";
			Log.d("SoundSquare", "addPoi, requestURL = " + requestURL);
			HttpGet request = new HttpGet(requestURL);

			HttpResponse response = client.execute(request);
			Log.d("SoundSquare", response.getStatusLine().toString());

			HttpEntity resEntity = response.getEntity();
			if (resEntity != null) {
				Log.d("SoundSquare", EntityUtils.toString(resEntity));
			}

			client.getConnectionManager().shutdown();
		} catch (Exception ex) {
			Log.d("SoundSquare", "Exception addPoi: " + ex.toString());
		}
	}

}
